<?php

namespace App\Console\Commands;

use Log;
use Carbon\Carbon;
use App\Models\Booking;
use Illuminate\Console\Command;
use App\Http\Controllers\apiController;

class GetBooking extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'getBooking:cron';

    private $apiController;

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    public function __construct(apiController $apiController)
    {
        parent::__construct();
        $this->apiController = $apiController;
    }

    /**
     * Execute the console command.
     */
    public function handle()
    {
        try {
            Log::info("Cron Is working!!!");
            $startDate = carbon::now()->toDateString();
            $endDate = carbon::now()-> addDay()->toDateString();
            $rollerBookings=[]; $middlewareBookings=[];

            $authToken = json_decode($this->apiController->authToken())->access_token;
            $curl = curl_init();
            curl_setopt_array($curl, array(
                CURLOPT_URL => "https://api.play.roller.app/data/bookingitems?startDate=$startDate&endDate=$endDate",
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'GET',
                CURLOPT_HTTPHEADER => array(
                'Authorization: Bearer '.$authToken
                ),
            ));
            $response = curl_exec($curl);
            curl_close($curl);
            
            $dataItems = json_decode($response)->items;
            foreach ($dataItems as $key => $dataItem) {
                $bookingId = $dataItem->bookingReference;
                array_push($rollerBookings,$bookingId);
            }
            $filteredBookingIds = array_unique($rollerBookings);
            
            $BookingDetails = Booking::all();
            foreach ($BookingDetails as $key => $BookingDetail) {
                $bookingDetailId = $BookingDetail->bookingReference; 
                array_push($middlewareBookings,$bookingDetailId);
            }

            $newCreatedBookings =  array_diff($filteredBookingIds,$middlewareBookings);
            $updatedAndCancelledBookings =  array_intersect($filteredBookingIds,$middlewareBookings);

            if (count($newCreatedBookings)!=0) {
            $bookingCreation = $this->apiController->searchBooking($authToken,$newCreatedBookings,'Create');
            }

            if (count($updatedAndCancelledBookings)!=0) {
                $bookingUpdate = $this->apiController->searchBooking($authToken,$updatedAndCancelledBookings,'Update');
            }

            if ( isset($bookingCreation->original['Error']) || isset($bookingUpdate->original['Error'])) {
                (isset($bookingCreation->original['Error']))?($error = $bookingCreation->original['Error']):($error = $bookingUpdate->original['Error']);
                Log::info($error);
            }
                Log::info("Cron is completed !!!!");
        } catch (\Throwable $th) {
            Log::info($th->getMessage());
        }
    }
}
