<?php

namespace App\Http\Controllers;

use Log;
use Carbon\Carbon;
use App\Models\Booking;
use App\Models\Tickets;
use App\Models\GateCode;
use App\Models\Direction;
use Illuminate\Http\Request;
use App\Models\BookingItems;
use App\Models\ResponseCode;
use App\Models\ProductDetails;
use Illuminate\Support\Facades\DB;
use App\Http\Controllers\DateTime;

/**
 * Class is using for validating Tickets in middleware using API's.
 * Class is using for getting Booking Details for middleware.
 */
class apiController extends Controller
{

  private $webhookController;

  public function __construct(webhookController $webhookController)
  {
    $this->webhookController = $webhookController;
    
  }

  /**
   * Validating Ticket API.
   *
   * @param json $request - Ticket Information.
   * 
   * @return json
   * 
   */
    public function ticketValidation(request $request){
      try {
            $AuthToken = $request->header('Authorization');
            //Request is Authenticated or not.
            if ($AuthToken == env('ALVARADO_PASSWORD')) {
              $ticket = $request->Ticket; 
              $ticketEntryGate= $request->DeviceNo;
              $ticketDate = $request->DateTime;
              $groupTickets=[]; $ticketsArray=[];
              $punchedBefore = Direction::where('ticket',$ticket)->whereIn('responseCode', [0, 10])->get()->first();

              $matchedTicket = Tickets::where('ticketId', $ticket)
              ->orWhere('customTicketId', $ticket)
              ->first();
              
              //Ticket is in middleware or not.
              if (isset($matchedTicket)) {
                $ticket = $matchedTicket->ticketId;
                $TicketUid = $matchedTicket->uniqueId;
                $Bookingstatus = Booking::where("uniqueId",$TicketUid)->get()->first()->status;
                // Is Ticket Cancelled
                if ($Bookingstatus == "Paid") {
                  $bookingItemId = $matchedTicket->bookingItemId;
                  $productId = BookingItems::where('bookingItemId',$bookingItemId)->first()->productId;
                  $OriginalGate =  ProductDetails::where("productId",$productId)->get()->first()->gateName;
                  $requestedGate = GateCode::where('gateCode',$ticketEntryGate)->get()->first();
                  (isset($requestedGate))?($requestedGate = $requestedGate->gateName):($requestedGate = null);
                  
                  //Location Checking.
                  if( (str_contains($OriginalGate,$requestedGate) && !empty($requestedGate)) ){
                    $bookingUid = $matchedTicket->uniqueId;
                    $name = Booking::where('uniqueId',$bookingUid)->first()->name;
                    $responseCode = 0;
        
                    //Date Checking.
                    $getTicketDate = date("Y-m-d", strtotime($ticketDate));
                    $originalBookingDate = BookingItems::where('bookingItemId',$bookingItemId)->first()->bookingDate;
                    $originalBookingEndingDate = BookingItems::where('bookingItemId',$bookingItemId)->first()->bookingEndDate;
                  
                    //checking is the ticket belongs to GoldPass/Platinum Pass
                    $ProductCategory = ProductDetails::where("productId",$productId)->get()->first()->productCategory;
                    if (($ProductCategory == "GoldPass") || $ProductCategory == "PlatinumPass") {
                      if (isset($punchedBefore->dayAndTime)) {
                        // additional timing for passes
                        $originalBookingDate = Carbon::parse($punchedBefore->dayAndTime)->toDateString();
                        $originalBookingEndingDate = Carbon::parse($punchedBefore->dayAndTime)->addDays()->toDateString();
                      }
                    }else{
                      if (isset($punchedBefore->dayAndTime)) {
                        $originalBookingDate = Carbon::parse($punchedBefore->dayAndTime)->toDateString();
                        $originalBookingEndingDate =  $originalBookingDate;
                      }
                    }
                    if (($getTicketDate >= $originalBookingDate) && ($getTicketDate <= $originalBookingEndingDate)) {
                      $bookingUid = $matchedTicket->uniqueId;
                      $name = Booking::where('uniqueId',$bookingUid)->first()->name;
                      $responseCode = 0;
                    }elseif (($getTicketDate > $originalBookingEndingDate)) {
                      (isset($punchedBefore->dayAndTime)?($responseCode=21):($responseCode=12) );
                    }else{
                      $responseCode = 12;}
                  }else{$responseCode = 16;}
                  if ($responseCode == 0) {
                    $voidData = tickets::where("ticketId",$ticket)->get()->first();
                    if ($voidData->void == config('GlobalConstants.voidOut')) {
                      $voidData->void = config('GlobalConstants.voidIn');
                      $voidData->save();
                    }elseif (($voidData->void == config('GlobalConstants.voidIn') && ($getTicketDate == $originalBookingEndingDate) )) {
                      if (($ProductCategory == "GoldPass") || $ProductCategory == "PlatinumPass") {
                        $voidData->void = config('GlobalConstants.voidTwo');
                        $voidData->save();                
                      }else{
                        $responseCode = 10;
                      }
                    }
                    elseif (($voidData->void == config('GlobalConstants.voidIn') || ($voidData->void == config('GlobalConstants.voidTwo') ))) {
                      $responseCode = 10;
                    }           
                  }

                  if (isset($punchedBefore->dayAndTime)) {
                    $redemptionApiStatus = true;
                  }else if (($responseCode == 0)||($responseCode == 10)) {
                    $ticketId = array("ticketId"=>$ticket);
                    array_push($ticketsArray,$ticketId);
                    $redemptionApiStatus = $this->redeemTicket($ticketsArray);
                  }else{$redemptionApiStatus = true;}


                  if (($redemptionApiStatus == 200) || (isset($punchedBefore->dayAndTime))) {
                    //storing user direction.
                    $direction = new Direction;
                    $direction->ticket = $ticket;
                    $direction->deviceNo = $ticketEntryGate;
                    $direction->gateDirection = $request->Direction;
                    $direction->uniqueCode = $matchedTicket->uniqueId;
                    $direction->bookingItemId = $matchedTicket->bookingItemId;
                    $direction->dayAndTime = $ticketDate;
                    $direction->responseCode = $responseCode;
                    $direction->save();
                  }else{
                    $responseCode = 07; //Unable To Redeme.
                  }
                  
                }elseif ($Bookingstatus == "Cancelled") {
                  $responseCode = 31; //Cancel
                }else{
                  $responseCode = 51; // Not Found
                }
              }else{
                $responseCode = 51; //Cancel
              }
      
              if (($responseCode == 0)||($responseCode == 10)) {
                $uniqueId=Tickets::where('ticketId',$ticket)->first()->uniqueId;
                if ($uniqueId) {
                  $bookingData = BookingItems::where('uniqueId',$uniqueId)->get();
                  foreach ($bookingData as $singleBookingData) {
                    $quantity = $singleBookingData->quantity;
                    $productId = $singleBookingData->productId;
                    $productName = ProductDetails::where('productId',$productId)->get()->first()->productType;
                    $OneTicket = [
                      "TicketType"=>$productName,
                      "Quantity"=>$quantity,
                    ];
                    $singleGroupTicket = json_decode(json_encode($OneTicket), false);
                    array_push($groupTickets,$singleGroupTicket);
                  }
                  if ((count($groupTickets) == 1) && ($groupTickets[0]->Quantity == 1)) {
                    $groupTickets=[];
                  }
                }
              }

                  // dd($responseCode);

              //checking for group tickets.
              $responseMessage = ResponseCode::where('code',$responseCode)->get()->first()->message;
              (($responseCode == 0)||($responseCode == 10))?($colorCode = "rgb(0,255,0)"):($colorCode = "rgb(255,0,0)");
              
              // Response collection
              $data =[
                "ticket"=>$ticket,
                "responseCode"=>$responseCode,
                "printSls"=>false,
                "eventDescription"=>"",
                "eventTime"=>"",
                "patronName"=>isset($name)?($name):"",
                "level"=>"",
                "section"=>"",
                "row"=>"",
                "seat"=>"",
                "line1"=>"",
                "line2"=>"",
                "line3"=>"",
                "line4"=>"",
                "line5"=>"",
                "ledColor"=>$colorCode,
                "responseMessage"=>$responseMessage,
                "photoUrl"=> "",
                "photoUri"=> "",
                "groupTicket"=> $groupTickets
              ];

              return $data;
            }else{
              return response()->json(['status' => 'Error','message' => 'Unauthorization Required'], 401);
            }
      } catch (\Throwable $th) {
          Log::info($th);
          return json_encode('Something Went wrong!!!');
      }    
    }

    /**
   * PING API.
   *
   * @param json $request - Device Information.
   * 
   * @return json
   * 
   */
  public function ping(Request $request){
    try {
      $AuthToken = $request->header('Authorization');
      $deviceNo = $request->DeviceNo;
      
      //Checking Authentication
      if ($AuthToken == env('ALVARADO_PASSWORD')) {
        $deviceFound = GateCode::where("gateCode",$deviceNo)->get();
        //if device found/connected
          if ((isset($deviceNo) && (!$deviceFound->isEmpty())) ) {
            $response=['status'=>'ok','message'=>'ping from '.$deviceNo];
            return $response;
          }else{return response()->json(['Error' => 'device Not Found.'],500);}
      }else{return response()->json(['status' => 'Error','message' => 'Authorization Required'], 401);}
    } catch (\Throwable $th) {
      Log::info($th);
      return json_encode('Something Went wrong!!!');
    }
    
  }

  /**
   * Group Ticket Validation API.
   *
   * @param json $request - Group Ticket Information.
   * 
   * @return json
   * 
   */
  public function groupTicketValidation(Request $request){
    try {
      $AuthToken = $request->header('Authorization');
      //Checking Authentication 
      if ($AuthToken == env('ALVARADO_PASSWORD')) {
        $ticketNo = $request->Ticket;
        $ticketDate = $request->DateTime;
        $ticketGroup = $request->GroupTicket;
        $productNames = [];
        $matchedTicket = Tickets::where('ticketId', $ticketNo)
        ->orWhere('customTicketId', $ticketNo)
        ->first();
        $ticketEntryGate = $request->DeviceNo;
        
        // Requested Ticket is present in DB or not.
        if (isset($matchedTicket)) {
          $ticketNo = $matchedTicket->ticketId;
          $TicketUid = $matchedTicket->uniqueId;
          $Bookingstatus = Booking::where("uniqueId",$TicketUid)->get()->first()->status;

          if ($Bookingstatus == "Paid") {
            $bookingItemId = $matchedTicket->bookingItemId;
            $productId = BookingItems::where('bookingItemId',$bookingItemId)->first()->productId;
            $OriginalGate =  ProductDetails::where("productId",$productId)->get()->first()->gateName;
            $requestedGate = GateCode::where('gateCode',$ticketEntryGate)->get()->first();
            (isset($requestedGate))?($requestedGate = $requestedGate->gateName):($requestedGate = null);
              
            //Location Checking.
            if( (!str_contains($OriginalGate,$requestedGate) && !empty($requestedGate)) ){
              $error = ["Status"=>"ERROR","message"=>"Invalid Location"];
              return $error;}
  
              //storing ticket direction 
              $direction = new Direction;
              $direction->ticket = $ticketNo;
              $direction->deviceNo = $ticketEntryGate;
              $direction->gateDirection = "IN";
              $direction->uniqueCode = $matchedTicket->uniqueId;
              $direction->bookingItemId = $matchedTicket->bookingItemId;
              $direction->responseCode = 0;
              $direction->dayAndTime = $ticketDate;
              $direction->save();
            
              $getTicketDate = date("Y-m-d", strtotime($ticketDate));
              $originalBookingDate = BookingItems::where('bookingItemId',$bookingItemId)->first()->bookingDate;
              $originalBookingEndingDate = BookingItems::where('bookingItemId',$bookingItemId)->first()->bookingEndDate;
              
              //Checking is ticket belongs form Gold/Platinum Pass
              $ProductCategory = ProductDetails::where("productId",$productId)->get()->first()->productCategory;
              if (($ProductCategory == "GoldPass") || $ProductCategory == "PlatinumPass") {
                $punchedBefore = Direction::where('ticket',$ticketNo)->where('responseCode',0)->get()->first();
                if (isset($punchedBefore->dayAndTime)) {
                  // additional timing for passes
                  $originalBookingDate = Carbon::parse($punchedBefore->dayAndTime)->toDateString();
                  $originalBookingEndingDate = Carbon::parse($punchedBefore->dayAndTime)->addDays()->toDateString();
                }
              }else{
                $punchedBefore = Direction::where('ticket',$ticketNo)->where('responseCode',0)->get()->first();
                if (isset($punchedBefore->dayAndTime)) {
                  // additional timing for passes
                  $originalBookingDate = Carbon::parse($punchedBefore->dayAndTime)->toDateString();
                  $originalBookingEndingDate = Carbon::parse($punchedBefore->dayAndTime)->toDateString();
                }               
              }
            // Checking Ticket Date.
            if (($getTicketDate >= $originalBookingDate) && ($getTicketDate <= $originalBookingEndingDate)) {
              $uniqueId=Tickets::where('ticketId',$ticketNo)->first()->uniqueId;
              if (isset($uniqueId)) {
                $bookingData = BookingItems::where('uniqueId',$uniqueId)->get();
                foreach ($bookingData as $singleBookingData) {
                  $productName = strtolower(ProductDetails::where('productId',$singleBookingData->productId)->get()->first()->productType);
                  array_push($productNames,$productName);
                  $groupTicketBookingId = $singleBookingData->bookingItemId;
                  $unVoidedTickets = Tickets::where('bookingItemId',$groupTicketBookingId)->count();
                  foreach ($ticketGroup as $singleTicket) {
                    $ticketType = strtolower($singleTicket['TicketType']);
                    if ($productName == $ticketType) {
                      if ($unVoidedTickets >= $singleTicket['Quantity']) {
                      }else{
                        $message = [
                          "Error"=>"Invalid Count for ".$productName,
                        ];
                        return $message;
                      }
                    }
                  }
                }
                foreach ($ticketGroup as $singleTicket) {
                  $productType = $singleTicket['TicketType'];
                  $name = strtolower($productType);              
                  if (!(in_array($name, $productNames))) {
                    $message = ["Error"=>"Invalid Product Type - ".$productType,];
                    return $message;
                  }
                }
                $response =[
                  'status' => 'OK',
                  'message' => "Group Ticket ".$ticketNo,
                ];
                return $response;
              }
            }else{
              return response()->json(['Error' => 'Date Not Matched.']);
            }
          } elseif ($Bookingstatus == "Cancelled") {
            return response()->json(['Error' => 'Ticket Cancelled.']);
          }else{
            return response()->json(['Error' => 'Ticket Not Found.']);
          }
        }else{
          return response()->json(['Error' => 'Ticket Not Found.']);
        }

      }else{
        return response()->json(['status' => 'Error','message' => 'Authorization Required'], 401);
      }  
    } catch (\Throwable $th) {
      Log::info($th);
      return json_encode('Something Went wrong!!!');
    }
  }

    /**
   * Force Ticket Validation API.
   *
   * @param json $request - Ticket Information.
   * 
   * @return json
   * 
   */
  public function forceValidation(Request $request){
    try {
      $AuthToken = $request->header('Authorization');
      //Request is Authenticated or not.
      if ($AuthToken == env('ALVARADO_PASSWORD')) {
        $ticket = $request->Ticket;
        $ticketEntryGate= $request->DeviceNo;
        $ticketDate = $request->DateTime;
        $gateDirection = $request->Direction;
        $matchedTicket = Tickets::where('ticketId', $ticket)
        ->orWhere('customTicketId', $ticket)
        ->first();
        $ticketsArray = [];
        
        //Ticket is valid or not.
        if (isset($matchedTicket)) {
          $ticket = $matchedTicket->ticketId;
          $TicketUid = $matchedTicket->uniqueId;
          $Bookingstatus = Booking::where("uniqueId",$TicketUid)->get()->first()->status;

          if ($Bookingstatus == "Paid") {
            $bookingItemId = $matchedTicket->bookingItemId;
            $productId = BookingItems::where('bookingItemId',$bookingItemId)->first()->productId;
            $OriginalGate =  ProductDetails::where("productId",$productId)->get()->first()->gateName;
            $requestedGate = GateCode::where('gateCode',$ticketEntryGate)->get()->first();            
            
            (isset($requestedGate))?($requestedGate = $requestedGate->gateName):($requestedGate = null);
            //Location Checking.
            if( (str_contains($OriginalGate,$requestedGate) && !empty($requestedGate)) ){
              $responseCode = 0;              
              //Date Checking.
              $getTicketDate = date("Y-m-d", strtotime($ticketDate));
              $originalBookingDate = BookingItems::where('bookingItemId',$bookingItemId)->first()->bookingDate;
              $originalBookingEndingDate = BookingItems::where('bookingItemId',$bookingItemId)->first()->bookingEndDate;  
              
              //Checking is ticket belongs form Gold/Platinum Pass
              $ProductCategory = ProductDetails::where("productId",$productId)->get()->first()->productCategory;
              $punchedBefore = Direction::where('ticket',$ticket)->whereIn('responseCode', [0, 10])->get()->first();
              if (($ProductCategory == "GoldPass") || $ProductCategory == "PlatinumPass") {
                if (isset($punchedBefore->dayAndTime)) {
                  // additional timing for passes
                  $originalBookingDate = Carbon::parse($punchedBefore->dayAndTime)->toDateString();
                  $originalBookingEndingDate = Carbon::parse($punchedBefore->dayAndTime)->addDays()->toDateString();
                }
              }else{
                if (isset($punchedBefore->dayAndTime)) {
                  // additional timing for passes
                  $originalBookingDate = Carbon::parse($punchedBefore->dayAndTime)->toDateString();
                  $originalBookingEndingDate =  $originalBookingDate;
                }
              }

              if (($getTicketDate >= $originalBookingDate) && ($getTicketDate <= $originalBookingEndingDate)) {
                $responseCode = 0;
              }elseif (($getTicketDate > $originalBookingEndingDate) && (isset($punchedBefore->dayAndTime))) {
                  $responseCode=21;
                  $message="WEB Used";
              }else{$responseCode = 12; $message="Invalid Date";}
    
            }else{$responseCode = 16; $message="Invalid Location";}
  
            if ($responseCode == 0) {
              $voidData = tickets::where("ticketId",$ticket)->get()->first();
              if ($voidData->void == config('GlobalConstants.voidOut')) {
                $voidData->void = config('GlobalConstants.voidIn');
                $voidData->save();
              }elseif (($voidData->void == config('GlobalConstants.voidIn') && ($getTicketDate == $originalBookingEndingDate) )) {
                if (($ProductCategory == "GoldPass") || $ProductCategory == "PlatinumPass") {
                  $voidData->void = config('GlobalConstants.voidTwo');
                  $voidData->save();                
                }else{
                  $responseCode = 10; $message = "WEB Reentry";
                }
              }
              elseif (($voidData->void == config('GlobalConstants.voidIn') || ($voidData->void == config('GlobalConstants.voidTwo') ))) {
                $responseCode = 10; $message = "WEB Reentry";
              }
            }
          }elseif ($Bookingstatus == "Cancelled") {
            $responseCode = 31; $message="Ticket Cancelled";
          }else{
            $responseCode = 51; $message = "Ticket Not Found !!!";
          }
        }else{$responseCode = 51; $message = "Ticket Not Found !!!";}

        $responseMessage = ResponseCode::where('code',$responseCode)->get()->first()->message;

        // Response collection
        if (($responseCode == 0) || ($responseCode == 10)) {
          if (isset($punchedBefore->dayAndTime)) {
            $status_code = 200;
          }else{
            $ticketId = array("ticketId"=>$ticket);
            array_push($ticketsArray,$ticketId);
            // Redeem Tickets from roller Database.
            $status_code = $this->redeemTicket($ticketsArray);
          }
          if ($status_code != 200 ) {
            $message = ["Error"=>"Unable To Redeeme Ticket!!!",];
            return $message;
          }else{
            $data =[
              "Status"=>"OK",
              "message"=>"Ticket ".$ticket." forced",
            ];

            //storing user direction 
            $direction = new Direction;
            $direction->ticket = $ticket;
            $direction->deviceNo = $ticketEntryGate;
            $direction->gateDirection = $gateDirection;
            $direction->uniqueCode = $matchedTicket->uniqueId;
            $direction->bookingItemId = $matchedTicket->bookingItemId;
            $direction->dayAndTime = $ticketDate;
            $direction->responseCode = $responseCode;
            $direction->save();
          }
        }else{
          $data =[
            "Status"=>"ERROR",
            "message"=>$message,
          ];      
        }
        return $data;
      }else{
        return response()->json(['status' => 'Error','message' => 'Authorization Required'], 401);
      }
    } catch (\Throwable $th) {
      Log::info($th);
      return json_encode('Something Went wrong!!!');
    }
  }

    /**
   * Device Sync API.
   *
   * @param json $request - Device Information.
   * 
   * @return json
   * 
   */
  public function deviceSync(Request $request){
    try {
      $AuthToken = $request->header('Authorization');

      if (($AuthToken == env('ALVARADO_PASSWORD'))) {          
          $deviceNo = $request->DeviceNo;
          $dateTime = $request->DateTime;
          $currentDate = Carbon::now()->format('Y-m-d\TH:i:s');
          $response =[
            'dateTime'=>$currentDate,
            'siteName'=>'Alvarado Test',
            'responseCodes'=>[
              ["code"=>0,"description"=>"WEB Valid"],
              ["code"=>10,"description"=>"WEB Reentry"],
              ["code"=>11,"description"=>"WEB Invalid Event"],
              ["code"=>12,"description"=>"WEB Invalid Date"],
              ["code"=>13,"description"=>"WEB Invalid Time"],
              ["code"=>14,"description"=>"WEB Invalid Day"],
              ["code"=>15,"description"=>"WEB No Access"],
              ["code"=>16,"description"=>"WEB Invalid Location"],
              ["code"=>21,"description"=>"WEB Used"],
              ["code"=>31,"description"=>"Ticket Cancelled"],
              ["code"=>32,"description"=>"Ticket Lost"],
              ["code"=>33,"description"=>"Ticket Stolen"],
              ["code"=>41,"description"=>"Already In"],
              ["code"=>42,"description"=>"Already Out"],
              ["code"=>43,"description"=>"Exit without Enter"],
              ["code"=>51,"description"=>"Not Found"],
              ["code"=>52,"description"=>"Rescan"],
              ["code"=>53,"description"=>"Invalid Mask"]
            ]
          ];
          return $response;
      }else{
        return response()->json(['status' => 'Error','message' => 'Authorization Required'], 401);
      }
      
    } catch (\Throwable $th) {
      Log::info($th);
      return json_encode('Something Went wrong!!!');
    }
  }

    /**
   * Void Ticket API.
   *
   * @param json $request - Ticket Information.
   * 
   * @return json
   * 
   */
  public function void(Request $request){
    try {
      $ticketNo = $request->Ticket;
      $matchedTicket = Tickets::where('ticketId', $ticketNo)
      ->orWhere('customTicketId', $ticketNo)
      ->first();
      $ticketNo = $matchedTicket->ticketId;

      $AuthToken = $request->header('Authorization');
      if (($AuthToken == env('ALVARADO_PASSWORD'))) {
        $ticket = Tickets::where('ticketId',$ticketNo)->first();
        $void = $ticket->void-1;
        ($void<=0)?($void=0):($void);
        if (isset($ticket)) {
            $ticket->void = $void;
            $ticket->save();
            $response=[
              'status'=>'OK',
              'message'=>'Ticket '.$ticketNo.' voided',
            ];
          return $response;
        }else{
          return response()->json(['status' => 'Error','message' => 'Ticket Not Found'], 200);
        }
      }else{return response()->json(['status' => 'Error','message' => 'Authorization Required'], 401);}
    } catch (\Throwable $th) {
      Log::info($th);
      return json_encode('Something Went wrong!!!');
    }
  }

  /**
   * Ticket Info API.
   *
   * @param json $request - Ticket Information.
   * 
   * @return json
   * 
   */
  public function ticketInfo(Request $request){  
    try {
      $AuthToken = $request->header('Authorization');
      //Authentication
      if (($AuthToken == env('ALVARADO_PASSWORD'))) {
        $ticket = $request->Ticket;
        $time = $request->DateTime;
        $deviceNo = $request->DeviceNo;
        $matchedTicket = Tickets::where('ticketId', $ticket)
        ->orWhere('customTicketId', $ticket)
        ->first();
        
        //Checking Ticket Validity
        if (isset($matchedTicket) && $matchedTicket != null) {
          $ticket = $matchedTicket->ticketId;
          $name = Booking::where("uniqueId",$matchedTicket->uniqueId)->first()->name; $responseCode = 0;
          if ($request->quantity) {
            $quantity = $request->quantity;
            
            $mainArray=[];
            $entryRecord = Direction::where('ticket', $ticket)->latest()->take($quantity)->get();
            foreach ($entryRecord as $singleEntryRecord) {
              $time =$singleEntryRecord->dayAndTime; $device=$singleEntryRecord->deviceNo;
              $responseEntryCode = $singleEntryRecord->responseCode;
              $array = ["responseCode"=>$responseEntryCode,"usedDateTime"=>$time,"usedLocation"=>$device];
              array_push($mainArray,$array);              
            }
            
            $response =[
              "ticket"=>$ticket,
              "responseCode"=>$responseCode,
              "patronName"=>$name,
              "level"=>"",
              "section"=>"",
              "row"=>"",
              "seat"=>"",
              "usedDateTime"=>$time,
              "usedLocation"=>$deviceNo,
              "usageData"=>$mainArray,
            ];
            return $response;

          }

        }else{$name = ""; $responseCode = 51;}

        $response =[
          "ticket"=>$ticket,
          "responseCode"=>$responseCode,
          "patronName"=>$name,
          "level"=>"",
          "section"=>"",
          "row"=>"",
          "seat"=>"",
          "usedDateTime"=>$time,
          "usedLocation"=>$deviceNo,
        ];
        return $response;
      }else{return response()->json(['status' => 'Error','message' => 'Authorization Required'], 401);}

    } catch (\Throwable $th) {
      Log::info($th);
      return json_encode('Something Went wrong!!!');
    }
  }

    /**
   * Authentication for Roller.
   *
   * @return $response ( Authentication Token )
   * 
   */
  public function authToken(){
    
    try {
      $curl = curl_init();

      curl_setopt_array($curl, array(
        CURLOPT_URL => env('ROLLER_URL').'/token',
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'POST',
        CURLOPT_POSTFIELDS => '{
          "client_id": "' . env('CLIENT_ID') . '",
          "client_secret": "' . env('CLIENT_SECRET') . '"
      }',
        CURLOPT_HTTPHEADER => array(
          'Content-Type: application/json'
        ),
      ));

      $response = curl_exec($curl);
      curl_close($curl);
      return $response;
    } catch (\Throwable $th) {
      Log::info($th);
    }
  }

  /**
   * Ticket Redemption from Roller.
   *  
   * @return $response ( Status )
   * 
   */
  protected function redeemTicket($ticketArray){
    try {
      $tickets = json_encode($ticketArray);
      $authToken = json_decode($this->authToken())->access_token;

      $curl = curl_init();
      curl_setopt_array($curl, array(
        CURLOPT_URL => env('ROLLER_URL').'/redemptions',
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'POST',
        CURLOPT_POSTFIELDS =>'{
        "tickets": '.$tickets.'
      }',
        CURLOPT_HTTPHEADER => array(
          'Content-Type: application/json',
          'Authorization: Bearer '.$authToken
        ),
      ));

      $response = curl_exec($curl);
      $status_code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
      curl_close($curl);
      return $status_code;
      
    } catch (\Throwable $th) {
      Log::info($th);
    }
  }

  /**
   * Getting ALL BookingItem Details Of One Day Who are Changed/Created.
   *  
   * @param $request - $request having the startDate and endDate.
   * 
   * @return json ( Status )
   * 
   */
  public function getData(Request $request){
    try {
        $startDate = $request->input('startDate');
        $endDate = $request->input('endDate');
        $rollerBookings=[]; $middlewareBookings=[];
  
        $authToken = json_decode($this->authToken())->access_token;
        $curl = curl_init();
        curl_setopt_array($curl, array(
          CURLOPT_URL => "https://api.play.roller.app/data/bookingitems?startDate=$startDate&endDate=$endDate",
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => '',
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 0,
          CURLOPT_FOLLOWLOCATION => true,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => 'GET',
          CURLOPT_HTTPHEADER => array(
            'Authorization: Bearer '.$authToken
          ),
        ));
        $response = curl_exec($curl);
        curl_close($curl);
        
        $dataItems = json_decode($response)->items;
        foreach ($dataItems as $key => $dataItem) {
          $bookingId = $dataItem->bookingReference;
          array_push($rollerBookings,$bookingId);
        }
        $filteredBookingIds = array_unique($rollerBookings);
        
        $BookingDetails = Booking::all();
        foreach ($BookingDetails as $key => $BookingDetail) {
          $bookingDetailId = $BookingDetail->bookingReference; 
          array_push($middlewareBookings,$bookingDetailId);
        }
  
        $newCreatedBookings =  array_diff($filteredBookingIds,$middlewareBookings);
        $updatedAndCancelledBookings =  array_intersect($filteredBookingIds,$middlewareBookings);

        if (count($newCreatedBookings)!=0) {
          $bookingCreation = $this->searchBooking($authToken,$newCreatedBookings,'Create');
        }
  
        if (count($updatedAndCancelledBookings)!=0) {
          $bookingUpdate = $this->searchBooking($authToken,$updatedAndCancelledBookings,'Update');
        }
  
        if ( isset($bookingCreation->original['Error']) || isset($bookingUpdate->original['Error'])) {
          (isset($bookingCreation->original['Error']))?($error = $bookingCreation->original['Error']):($error = $bookingUpdate->original['Error']);
          return response()->json(["Error"=>$error]);
        }else{
          return response()->json(["Message"=>"Data is Stored / Updated"]);
        }
    } catch (\Throwable $th) {
      Log::info($th->getMessage());
      return response()->json(["Error"=>$th->getMessage()]);
    }
  }
   
  /**
    * Used For Booking Search
    *  
    * @param $authToken - AuthenticationToken
    * @param $Ids - Id of Booking
    * @param $type - Type of Booking
    * 
    * @return json ( Status )
    * 
  */
  Public function searchBooking($authToken,$Ids,$type){
    try {
      foreach ($Ids as $key => $Id) {
        $curl = curl_init();
          curl_setopt_array($curl, array(
            CURLOPT_URL => "https://api.play.roller.app/bookings/$Id",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'GET',
            CURLOPT_HTTPHEADER => array(
              'Content-Type: application/json',
              'Authorization: Bearer '.$authToken
            ),
          ));
        $response = curl_exec($curl);
        curl_close($curl);
        $response = json_decode($response, true);

        if ($type == 'Create') {
          $result = $this->webhookController->createBooking($response);
        }else{
          $result = $this->webhookController->UpdateBookingData($response);
        }
      }

      if (isset($result->original['Error'])) {
        return $result;
      }
    } catch (\Throwable $th) {
      return response()->json(["Error"=>$th->getMessage()]);
      Log::info($th->getMessage());
    }
  }

}