<?php

namespace App\Http\Controllers;

use Log;
use Mail;
use App\Models\Booking;
use App\Models\Tickets;
use Illuminate\Http\Request;
use App\Models\BookingItems;
use App\Models\WebhookError;
use Illuminate\Support\Facades\DB;
use App\Mail\bookingNotStored;

/**
 * Class is store/modify DB according to Booking functionalities using Webhooks.
 * Class is use for Storing Booking Data in middleware.
 * Class is use for BookingCancelation in middleware.
*/
class webhookController extends Controller
{

  /**
   * Getting Webhook From Roller For Creating Bookings.
   *
   * @param json $request - booking data.
   * 
   * @return Log - Status
   * 
   */
  public function bookingPayload(request $request){
    try {
      $data = $request->all();
      Log::info($data);
      $bookingData = $data['data']['booking'];
      $authKey = $request->header('x-roller-apikey');
      if ($authKey == env('CLIENT_SECRET')) {
        $result = $this->createBooking($bookingData);
        (isset($result->original['Error']))?(Log::info($result->original['Error'])):"";
      }
      else{
        Log::info(json_encode('UnAuthorized Request'));
      }
    } catch (\Throwable $th) {
      Log::info($th->getMessage());
    }
  
  }

  /**
   *  Getting Webhook From Roller For Cancel Bookings.
   *
   * @param json $request - booking data.
   * 
   * @return Log - Status
   * 
   */
  public function cancleBooking(request $request){
    try {
        $data = $request->all();
        $bookingData = $data['data']['booking'];
        $authKey = $request->header('x-roller-apikey');
        if ($authKey == env('CLIENT_SECRET')) {
          $result = $this->UpdateBookingData($bookingData);
          (isset($result->original['Error']))?(Log::info($result->original['Error'])):"";
        }else{
            Log::info(json_encode('UnAuthorized Request'));
        }
    } catch (\Throwable $th) {
      Log::info($th->getMessage());
      // $this->errorTackling($th->getMessage(),$request->all(),"cancelled");
    }
  }

   /**
   * Getting Webhook From Roller For Update Bookings.
   *
   * @param json $request - booking data.
   * 
   * @return Log - Status
   * 
   */
  public function updateBooking(request $request){
    try {  
      $data = $request->all();
      Log::info($data);
      $bookingData = $data['data']['booking'];
      $authKey = $request->header('x-roller-apikey');
      
      if ($authKey == env('CLIENT_SECRET')) {
        $result = $this->UpdateBookingData($bookingData);
        (isset($result->original['Error']))?(Log::info($result->original['Error'])):"";
      }
    } catch (\Throwable $th) {
      DB::rollback();
      Log::info($th->getMessage());
      // $this->errorTackling($th->getMessage(),$request->all(),"updated");
    }
  }

  /**
   * Update webhook Errors in Middleware Database and Sent a mail to admin.
   *
   * @param json $error - Error we faced.
   * @param json $payload - Payload we get from request.
   * @param json $type - Type of Booking.
   * 
   * @return Log - Status
   * 
   */
  private function errorTackling($error,$bookingData,$type){
    try {
      $webhookError = new WebhookError;
      $bookingId = $bookingData['bookingReference'];
      $webhookError->booking_id = $bookingId;
      $webhookError->payload = json_encode($bookingData);
      $webhookError->error = $error;
      $webhookError->type = $type;
      $webhookError->save();
      $details = [
        'error' => $error,
        'bookingId'=>$bookingId,
        'type'=>$type
      ];
      Mail::to('abc00001@yopmail.com')->send(new bookingNotStored($details));
      Log::info("mail sent from middleware");
    } catch (\Throwable $th) {
      Log::info($th->getMessage());
    }
  }

   /**
   * Use for Creating booking in Middleware Database.
   *
   * @param Array $request - payload for Booking
   * 
   * @return Boolean - true/false.
   * 
   */
  Public function createBooking($request){
    try {
      $bookingData = $request; $bookingItem = $bookingData['items'];
      DB::beginTransaction();
      //For Booking Table  
      $booking = new Booking;
      $booking->bookingReference = $bookingData['bookingReference'];
      $booking->uniqueId = $bookingData['uniqueId'];
      $booking->source = $bookingData['source'];
      $booking->status = $bookingData['status'];
      $booking->name = $bookingData['name'];
      $booking->createdDate = $bookingData['createdDate'];
      $booking->customerId = (isset($bookingData['customerId']))?($bookingData['customerId']):Null;
      // $booking->customerId = $bookingData['customerId'];
      $booking->comments = $bookingData['comments'];
      $booking->total = $bookingData['total'];
      $booking->remainder = $bookingData['remainder'];
      $booking->fees = $bookingData['fees'];
      $booking->discount = (isset($bookingData['discount']))?($bookingData['discount']):0;
      $booking->save();

      // //For Booking-Items Table
      foreach ($bookingItem as $key => $singleBookingIteam) {
        $bookingItems = new BookingItems;
        $bookingItems->uniqueId = $bookingData['uniqueId'];
        $bookingItems->bookingItemId = $singleBookingIteam['bookingItemId'];
        $bookingItems->productId = $singleBookingIteam['productId'];
        $bookingItems->quantity = $singleBookingIteam['quantity'];
        $bookingItems->bookingDate = (isset($singleBookingIteam['bookingDate']))?($singleBookingIteam['bookingDate']):"";
        $bookingItems->bookingEndDate = (isset($singleBookingIteam['bookingEndDate']))?($singleBookingIteam['bookingEndDate']):"";
        // group size
        $bookingItems->sessionStartTime = (isset($singleBookingIteam['sessionStartTime']))?($singleBookingIteam['sessionStartTime']):"";
        $bookingItems->sessionEndTime = (isset($singleBookingIteam['sessionEndTime']))?($singleBookingIteam['sessionEndTime']):"";
        $bookingItems->cost = $singleBookingIteam['cost'];
        $bookingItems->discount = (isset($singleBookingIteam['discount']))?($singleBookingIteam['discount']):0;
        $bookingItems->save();

        $ticketsForBooking =  $singleBookingIteam['tickets'];
        foreach ($ticketsForBooking as $key => $ticket) {
          // For Tickets table
          $tickets = new Tickets;
          $tickets->uniqueId = $bookingData['uniqueId'];
          $tickets->bookingItemId = $singleBookingIteam['bookingItemId'];
          $tickets->ticketId = $ticket['ticketId' ];
          $tickets->customTicketId = (isset($ticket['customTicketId']))?($ticket['customTicketId']):"";
          $tickets->ticketHolderName = $ticket['ticketHolderName']; 
          $tickets->membershipStatus = (isset($ticket['membershipStatus']))?($ticket['membershipStatus']):"";
          $tickets->photoUrl = (isset($ticket['photoUrl']))?($ticket['photoUrl']):"";
          $tickets->customerId = (isset($ticket['customerId']))?($ticket['customerId']):"";
          $tickets->save();
        }
      }
      DB::commit();
      Log::info(json_encode('Booking Payload is Stored Successfully'));
      return True;
    } catch (\Throwable $th) {
      DB::rollback();
      Log::info($th->getMessage());
      $this->errorTackling($th->getMessage(),$request,"created");
      return response()->json(["Error"=>$th->getMessage()]);
    }
  }

  /**
   * Use for Update/Cancel booking in Middleware Database.
   *
   * @param Array $request - payload for Booking
   * 
   * @return Boolean - true/false.
   * 
   */
  Public function UpdateBookingData($request){
    try {
      $bookingData = $request;
      $bookingItem = $bookingData['items'];
      $bookingID = $bookingData['bookingReference'];
      //For Booking Table  
      DB::beginTransaction();        
      $booking = Booking::where("bookingReference",$bookingID)->get()->first();
      $uniqueId = $booking->uniqueId;
      $booking->source = $bookingData['source'];
      $booking->status = $bookingData['status'];
      $booking->name = (isset($bookingData['name']))?($bookingData['name']):Null;
      // $booking->name = $bookingData['name'];
      $booking->createdDate = $bookingData['createdDate'];
      $booking->customerId = (isset($bookingData['customerId']))?($bookingData['customerId']):Null;
      $booking->comments = $bookingData['comments'];
      $booking->total = $bookingData['total'];
      $booking->remainder = $bookingData['remainder'];
      $booking->fees = $bookingData['fees'];
      $booking->discount = (isset($bookingData['discount']))?($bookingData['discount']):0;
      if ($bookingData['status'] == "Cancelled") {
        $booking->isCancelled = config('GlobalConstants.Cancelled');
      }
      $booking->save();
      
      $bookingItems = BookingItems::where("uniqueId",$uniqueId)->delete();
      $ticket = Tickets::where("uniqueId",$uniqueId)->delete();

      foreach ($bookingItem as $key => $singleBookingIteam) {
        $bookingItems = new BookingItems;
        $bookingItems->uniqueId = $bookingData['uniqueId'];
        $bookingItems->bookingItemId = $singleBookingIteam['bookingItemId'];
        $bookingItems->productId = $singleBookingIteam['productId'];
        $bookingItems->quantity = $singleBookingIteam['quantity'];
        $bookingItems->bookingDate = (isset($singleBookingIteam['bookingDate']))?($singleBookingIteam['bookingDate']):"";
        $bookingItems->bookingEndDate = (isset($singleBookingIteam['bookingEndDate']))?($singleBookingIteam['bookingEndDate']):"";
        $bookingItems->sessionStartTime = (isset($singleBookingIteam['sessionStartTime']))?($singleBookingIteam['sessionStartTime']):"";
        $bookingItems->sessionEndTime = (isset($singleBookingIteam['sessionEndTime']))?($singleBookingIteam['sessionEndTime']):"";
        $bookingItems->cost = $singleBookingIteam['cost'];
        $bookingItems->discount = (isset($singleBookingIteam['discount']))?($singleBookingIteam['discount']):0;
        // $bookingItems->discount = $singleBookingIteam['discount'];
        $bookingItems->save();

          $ticketsForBooking =  $singleBookingIteam['tickets'];
          foreach ($ticketsForBooking as $key => $ticket) {
            $tickets = new Tickets;
            $tickets->uniqueId = $bookingData['uniqueId'];
            $tickets->bookingItemId = $singleBookingIteam['bookingItemId'];
            $tickets->ticketId = $ticket['ticketId' ];
            $tickets->customTicketId = (isset($ticket['customTicketId']))?($ticket['customTicketId']):"";
            $tickets->ticketHolderName = $ticket['ticketHolderName']; 
            $tickets->membershipStatus = (isset($ticket['membershipStatus']))?($ticket['membershipStatus']):"";
            $tickets->photoUrl = (isset($ticket['photoUrl']))?($ticket['photoUrl']):"";
            $tickets->customerId = (isset($ticket['customerId']))?($ticket['customerId']):"";
            $tickets->save();
          }
      }
      DB::commit();
      Log::info(json_encode('Booking is updated!!!'));
      return True;
    } catch (\Throwable $th) {
      DB::rollback();
      ($request['status'] == "Cancelled")?($type = "cancelled"):($type = "updated");
      Log::info($th->getMessage());
      $this->errorTackling($th->getMessage(),$request,$type);
      return response()->json(["Error"=>$th->getMessage()]);
    }
  }

}